#-----------------------------------------------------------------------
# Aplicar o método da aceitação e rejeição para gerar números da
# trapeizoidal usando a densidade seno.

# Função densidade, f(x), da trapezoidal.
fx <- function(x, a = 0, b = pi/3, c = 2 * pi/3, d = pi) {
    k <- 2/(d + c - b - a)
    y <- 0 +
        (x - a)/(b - a) * (x >= a & x < b) +
        (x >= b & x < c) +
        (d - x)/(d - c) * (x >= c & x <= d)
    return(k * y)
}

curve(fx, 0, pi)

# Função densidade, g(x), da seno.
gx <- function(x) {
    sin(x)/2 * (x >= 0 & x <= pi)
}

curve(gx, 0, pi)

# Gerador de números aleatórios da seno, g(x).
rgx <- function(n) {
    u <- runif(n)
    x <- acos(1 - 2 * u)
    return(x)
}

curve(fx, 0, pi)
curve(gx, add = TRUE, col = 2)

curve(fx(x)/gx(x), 0, pi)
abline(h = 1.1, col = 2)

c <- 1.1

curve(fx, 0, pi, ylim = c(0, 0.6))
curve(c * gx(x), add = TRUE, col = 2)

# Função para gerar números aleatórios da trapezoidal pelo método da
# aceitação e rejeição.
rtrap <- function(n) {
    x <- numeric(n)
    i <- 1
    while (i <= n) {
        y <- rgx(n = 1)
        u <- runif(n = 1)
        if (u <= fx(y)/(c * gx(y))) {
            x[i] <- y
            i <- i + 1
        }
    }
    return(x)
}

x <- rtrap(n = 1000)
plot(density(x))
curve(fx, add = TRUE, col = 2)

#-----------------------------------------------------------------------
# Ilustração do método.

curve(fx, 0, pi, ylim = c(0, 0.6))
curve(c * gx(x), add = TRUE, col = 2)
legend("topleft",
       legend = c("Aceitos", "Rejeitados"), col = c(4, 2),
       pch = 1, bty = "n")
n <- 100
i <- 1
while (i <= n) {
    Sys.sleep(0.05)
    y <- rgx(n = 1)
    u <- runif(n = 1)
    h <- c * gx(y)
    if (u <= fx(y)/h) {
        points(x = y, y = u * gx(y), col = 4)
        rug(x = y, col = 4)
        i <- i + 1
    } else {
        points(x = y, y = u * h, col = 2)
    }
}

#-----------------------------------------------------------------------
